package com.sun.org.apache.bcel.internal.util;

import java.io.File;
import java.util.Iterator;
import java.nio.file.DirectoryStream;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.List;
import java.nio.file.Path;
import java.util.Map;
import java.net.URL;
import java.nio.file.Paths;
import java.util.Collections;
import java.io.IOException;
import java.nio.file.FileSystems;
import java.net.URI;
import java.nio.file.FileSystem;
import java.net.URLClassLoader;
import java.io.Closeable;

public class ModularRuntimeImage implements Closeable
{
    static final String MODULES_PATH;
    static final String PACKAGES_PATH;
    private final URLClassLoader classLoader;
    private final FileSystem fileSystem;
    
    public ModularRuntimeImage() throws IOException {
        this(null, FileSystems.getFileSystem(URI.create("jrt:/")));
    }
    
    public ModularRuntimeImage(String javaHome) throws IOException {
        Map<String, ?> emptyMap = Collections.emptyMap();
        Path jrePath = Paths.get(javaHome, new String[0]);
        Path jrtFsPath = jrePath.resolve("lib").resolve("jrt-fs.jar");
        this.classLoader = new URLClassLoader(new URL[] { jrtFsPath.toUri().toURL() });
        this.fileSystem = FileSystems.newFileSystem(URI.create("jrt:/"), emptyMap, this.classLoader);
    }
    
    private ModularRuntimeImage(URLClassLoader cl, FileSystem fs) {
        this.classLoader = cl;
        this.fileSystem = fs;
    }
    
    @Override
    public void close() throws IOException {
        if (this.classLoader != null) {
            if (this.classLoader != null) {
                this.classLoader.close();
            }
            if (this.fileSystem != null) {
                this.fileSystem.close();
            }
        }
    }
    
    public List<Path> list(Path dirPath) throws IOException {
        List<Path> list = new ArrayList();
        try (final DirectoryStream<Path> ds = Files.newDirectoryStream(dirPath)) {
            Iterator<Path> iterator = ds.iterator();
            while (iterator.hasNext()) {
                list.add(iterator.next());
            }
        }
        return list;
    }
    
    public List<Path> list(String dirName) throws IOException {
        return this.list(this.fileSystem.getPath(dirName, new String[0]));
    }
    
    public List<Path> modules() throws IOException {
        return this.list(ModularRuntimeImage.MODULES_PATH);
    }
    
    public List<Path> packages() throws IOException {
        return this.list(ModularRuntimeImage.PACKAGES_PATH);
    }
    
    public FileSystem getFileSystem() {
        return this.fileSystem;
    }
    
    static {
        MODULES_PATH = File.separator + "modules";
        PACKAGES_PATH = File.separator + "packages";
    }
}
